@@ -2,24 +2,14 @@ from __future__ import annotations
 
 from collections import deque
 from contextlib import contextmanager
-from enum import Enum
-from typing import (
-    Mapping,
-    Union,
-    Optional,
-    Dict,
-    Iterator,
-    ContextManager,
-    Hashable,
-    TYPE_CHECKING,
-)
+from functools import cached_property
+from typing import Any, ContextManager, Dict, Hashable, Iterator, Mapping, Optional, TYPE_CHECKING, Type, Union
 from uuid import uuid4
 
 from jschon.exceptions import JSONSchemaError
-from jschon.json import JSON, AnyJSONCompatible
+from jschon.json import JSON, JSONCompatible
 from jschon.jsonpointer import JSONPointer
 from jschon.uri import URI
-from jschon.utils import tuplify
 
 if TYPE_CHECKING:
     from jschon.catalog import Catalog
@@ -28,7 +18,6 @@ if TYPE_CHECKING:
 __all__ = [
     'JSONSchema',
     'Scope',
-    'OutputFormat',
 ]
 
 
@@ -37,9 +26,9 @@ class JSONSchema(JSON):
 
     def __init__(
             self,
-            value: Union[bool, Mapping[str, AnyJSONCompatible]],
+            value: Union[bool, Mapping[str, JSONCompatible]],
             *,
-            catalog: Catalog = None,
+            catalog: Union[str, Catalog] = 'catalog',
             session: Hashable = 'default',
             uri: URI = None,
             metaschema_uri: URI = None,
@@ -50,10 +39,7 @@ class JSONSchema(JSON):
         schema-compatible `value`.
 
         :param value: a schema-compatible Python object
-        :param catalog: the catalog in which the schema is cached;
-            omitting this parameter has the same effect as setting it
-            to the default catalog, i.e. that created by
-            `jschon.create_catalog(..., default=True)`
+        :param catalog: the catalog in which the schema is cached
         :param session: a session identifier, identifying which session
             cache to put the schema in
         :param uri: the URI identifying the schema; an ``"$id"`` keyword
@@ -67,12 +53,8 @@ class JSONSchema(JSON):
         """
         from jschon.catalog import Catalog
 
-        if catalog is None:
-            if (catalog := Catalog.get_default()) is None:
-                raise JSONSchemaError("catalog not given and default catalog not found")
-
-        if uri is not None:
-            catalog.add_schema(uri, self, session=session)
+        if not isinstance(catalog, Catalog):
+            catalog = Catalog.get_catalog(catalog)
 
         self.catalog: Catalog = catalog
         """The catalog in which the schema is cached."""
@@ -81,6 +63,9 @@ class JSONSchema(JSON):
         """A session identifier, identifying which session
         cache to use for the schema."""
 
+        if uri is not None:
+            catalog.add_schema(uri, self, session=session)
+
         self._uri: Optional[URI] = uri
         self._metaschema_uri: Optional[URI] = metaschema_uri
 
@@ -115,7 +100,7 @@ class JSONSchema(JSON):
             self.type = "boolean"
             self.data = value
 
-        elif isinstance(value, Mapping) and all(isinstance(k, str) for k in value):
+        elif isinstance(value, Mapping):
             self.type = "object"
             self.data = {}
 
@@ -141,7 +126,7 @@ class JSONSchema(JSON):
         else:
             raise TypeError(f"{value=} is not JSONSchema-compatible")
 
-    def _bootstrap(self, value: Mapping[str, AnyJSONCompatible]) -> None:
+    def _bootstrap(self, value: Mapping[str, JSONCompatible]) -> None:
         from jschon.vocabulary.core import IdKeyword, SchemaKeyword, VocabularyKeyword
         boostrap_kwclasses = {
             "$id": IdKeyword,
@@ -172,7 +157,7 @@ class JSONSchema(JSON):
     @staticmethod
     def _resolve_dependencies(kwclasses: Dict[str, KeywordClass]) -> Iterator[KeywordClass]:
         dependencies = {
-            kwclass: [depclass for dep in tuplify(kwclass.depends)
+            kwclass: [depclass for dep in kwclass.depends
                       if (depclass := kwclasses.get(dep))]
             for kwclass in kwclasses.values()
         }
@@ -203,7 +188,7 @@ class JSONSchema(JSON):
             when invoking this method recursively
         """
         if scope is None:
-            scope = Scope(self)
+            scope = Scope(self, instance)
 
         if self.data is True:
             pass
@@ -213,7 +198,7 @@ class JSONSchema(JSON):
 
         else:
             for key, keyword in self.keywords.items():
-                if keyword.can_evaluate(instance):
+                if not keyword.static and keyword.can_evaluate(instance):
                     with scope(instance, key, self) as subscope:
                         keyword.evaluate(instance, subscope)
 
@@ -225,7 +210,7 @@ class JSONSchema(JSON):
 
         return scope
 
-    @property
+    @cached_property
     def parentschema(self) -> Optional[JSONSchema]:
         """The containing :class:`JSONSchema` instance.
         
@@ -329,47 +314,57 @@ class Scope:
     def __init__(
             self,
             schema: JSONSchema,
+            instance: JSON,
             *,
-            key: str = None,
-            path: JSONPointer = None,
-            relpath: JSONPointer = None,
-            instpath: JSONPointer = None,
             parent: Scope = None,
+            key: str = None,
     ):
         self.schema: JSONSchema = schema
-        self.key: Optional[str] = key
-        self.path: JSONPointer = path or JSONPointer()
-        self.relpath: JSONPointer = relpath or JSONPointer()
-        self.instpath: JSONPointer = instpath or JSONPointer()
+        self.instance: JSON = instance
         self.parent: Optional[Scope] = parent
+        self.key: Optional[str] = key
         self.children: Dict[JSONPointer, Dict[str, Scope]] = {}
-        self.annotation: AnyJSONCompatible = None
+        self.annotation: JSONCompatible = None
         self.error: Optional[str] = None
         self._valid = True
         self._assert = True
         self._discard = False
         self._refschema: Optional[JSONSchema] = None
 
+        if parent is None:
+            self.path = JSONPointer()
+            self.relpath = JSONPointer()
+            self._globals = {}
+        else:
+            self.path = parent.path / key
+            self.relpath = parent.relpath / key if schema is parent.schema else JSONPointer((key,))
+            self._globals = None
+
     @contextmanager
-    def __call__(self, instance: JSON, key: str, schema: JSONSchema = None) -> ContextManager[Scope]:
+    def __call__(
+            self,
+            instance: JSON,
+            key: str,
+            schema: JSONSchema = None,
+            *,
+            cls: Type[Scope] = None,
+    ) -> ContextManager[Scope]:
         """Yield a subscope of the current scope, for evaluating `instance`.
         Descend down the evaluation path by `key`, into `schema` if given, or
-        within the schema of the current scope otherwise."""
-        path = self.path / key
-        schema = schema or self.schema
-        if schema is self.schema:
-            relpath = self.relpath / key
-        else:
-            relpath = JSONPointer((key,))
+        within the schema of the current scope otherwise.
+
+        Extension keywords may provide a custom Scope class via `cls`, which
+        is applied to all nodes within the yielded subtree.
+        """
+        if schema is None:
+            schema = self.schema
 
         self.children.setdefault(instance_path := instance.path, {})
-        self.children[instance_path][key] = (child := Scope(
+        self.children[instance_path][key] = (child := (cls or self.__class__)(
             schema,
-            key=key,
-            path=path,
-            relpath=relpath,
-            instpath=instance_path,
+            instance,
             parent=self,
+            key=key,
         ))
 
         try:
@@ -378,13 +373,25 @@ class Scope:
             if child._discard:
                 del self.children[instance_path][key]
 
+    @cached_property
+    def globals(self) -> Dict:
+        root = self
+        while root.parent is not None:
+            root = root.parent
+        return root._globals
+
+    @cached_property
+    def schema_node(self) -> JSON:
+        """Return the schema node associated with this scope."""
+        return self.relpath.evaluate(self.schema)
+
     def sibling(self, instance: JSON, key: str) -> Optional[Scope]:
         try:
             return self.parent.children[instance.path][key] if self.parent else None
         except KeyError:
             return None
 
-    def annotate(self, value: AnyJSONCompatible) -> None:
+    def annotate(self, value: JSONCompatible) -> None:
         """Set an annotation on the scope."""
         self.annotation = value
 
@@ -460,42 +467,28 @@ class Scope:
             if instance is None or instance.path == instance_path:
                 yield from keyword_scopes.values()
 
-    def collect_annotations(self, instance: JSON = None, key: str = None) -> Iterator[AnyJSONCompatible]:
+    def collect_annotations(self, instance: JSON = None, key: str = None) -> Iterator[JSONCompatible]:
         """Return an iterator over annotations produced in this subtree,
         optionally filtered by instance and/or keyword."""
         if self._valid and not self._discard:
             if self.annotation is not None and \
                     (key is None or key == self.key) and \
-                    (instance is None or instance.path == self.instpath):
+                    (instance is None or instance.path == self.instance.path):
                 yield self.annotation
             for child in self.iter_children():
                 yield from child.collect_annotations(instance, key)
 
-    def output(self, format: OutputFormat) -> Dict[str, AnyJSONCompatible]:
-        """Return an output dictionary formatted in accordance with the
-        JSON Schema specification of the given output `format`."""
-        from jschon.output import OutputFormatter
+    def output(self, format: str, **kwargs: Any) -> JSONCompatible:
+        """Return the evaluation result in the specified `format`.
 
-        if format == OutputFormat.FLAG:
-            return OutputFormatter.flag(self)
-
-        if format == OutputFormat.BASIC:
-            return OutputFormatter.basic(self)
-
-        if format == OutputFormat.DETAILED:
-            return OutputFormatter.detailed(self)
-
-        if format == OutputFormat.VERBOSE:
-            return OutputFormatter.verbose(self)
-
-        raise NotImplementedError
+        :param format: The format of the returned document. The default
+            :class:`~jschon.output.JSONSchemaOutputFormatter` supports
+            the ``flag``, ``basic``, ``detailed`` and ``verbose`` output
+            formats, as defined by the JSON Schema specification.
+        :param kwargs: Additional keyword arguments to pass to the output
+            formatter.
+        """
+        return self.schema.catalog.output_formatter.create_output(self, format, **kwargs)
 
     def __str__(self) -> str:
         return str(self.path)
-
-
-class OutputFormat(str, Enum):
-    FLAG = 'flag'
-    BASIC = 'basic'
-    DETAILED = 'detailed'
-    VERBOSE = 'verbose'
