@@ -1,30 +1,45 @@
-from __future__ import annotations
+from typing import Any, Dict
 
-from typing import Dict
-
-from jschon.json import AnyJSONCompatible
+from jschon.json import JSONCompatible
 from jschon.jsonschema import Scope
 
 __all__ = [
     'OutputFormatter',
+    'JSONSchemaOutputFormatter',
 ]
 
 
 class OutputFormatter:
 
+    def create_output(self, scope: Scope, format: str, **kwargs: Any) -> JSONCompatible:
+        raise NotImplementedError
+
+
+class JSONSchemaOutputFormatter(OutputFormatter):
+
+    def create_output(self, scope: Scope, format: str, **kwargs: Any) -> JSONCompatible:
+        if format == 'flag':
+            return self._flag(scope)
+        if format == 'basic':
+            return self._basic(scope)
+        if format == 'detailed':
+            return self._detailed(scope)
+        if format == 'verbose':
+            return self._verbose(scope)
+
     @staticmethod
-    def flag(scope: Scope) -> Dict[str, AnyJSONCompatible]:
+    def _flag(scope: Scope) -> Dict[str, JSONCompatible]:
         return {
             "valid": scope.valid
         }
 
     @staticmethod
-    def basic(scope: Scope) -> Dict[str, AnyJSONCompatible]:
+    def _basic(scope: Scope) -> Dict[str, JSONCompatible]:
         def visit(node: Scope):
             if node.valid is valid:
                 if (msgval := getattr(node, msgkey)) is not None:
                     yield {
-                        "instanceLocation": str(node.instpath),
+                        "instanceLocation": str(node.instance.path),
                         "keywordLocation": str(node.path),
                         "absoluteKeywordLocation": str(node.absolute_uri),
                         msgkey: msgval,
@@ -42,10 +57,10 @@ class OutputFormatter:
         }
 
     @staticmethod
-    def detailed(scope: Scope) -> Dict[str, AnyJSONCompatible]:
+    def _detailed(scope: Scope) -> Dict[str, JSONCompatible]:
         def visit(node: Scope):
             result = {
-                "instanceLocation": str(node.instpath),
+                "instanceLocation": str(node.instance.path),
                 "keywordLocation": str(node.path),
                 "absoluteKeywordLocation": str(node.absolute_uri),
                 childkey: [visit(child) for child in node.iter_children()
@@ -66,7 +81,7 @@ class OutputFormatter:
 
         return {
             "valid": valid,
-            "instanceLocation": str(scope.instpath),
+            "instanceLocation": str(scope.instance.path),
             "keywordLocation": str(scope.path),
             "absoluteKeywordLocation": str(scope.absolute_uri),
             childkey: [visit(child) for child in scope.iter_children()
@@ -74,11 +89,11 @@ class OutputFormatter:
         }
 
     @staticmethod
-    def verbose(scope: Scope) -> Dict[str, AnyJSONCompatible]:
+    def _verbose(scope: Scope) -> Dict[str, JSONCompatible]:
         def visit(node: Scope):
             result = {
                 "valid": (valid := node.valid),
-                "instanceLocation": str(node.instpath),
+                "instanceLocation": str(node.instance.path),
                 "keywordLocation": str(node.path),
                 "absoluteKeywordLocation": str(node.absolute_uri),
             }
