@@ -49,14 +49,12 @@ class RecursiveRefKeyword_2019_09(Keyword):
 
 class RecursiveAnchorKeyword_2019_09(Keyword):
     key = "$recursiveAnchor"
-
-    def can_evaluate(self, instance: JSON) -> bool:
-        return False
+    static = True
 
 
 class ItemsKeyword_2019_09(Keyword, Applicator, ArrayApplicator):
     key = "items"
-    types = "array"
+    types = "array",
 
     def evaluate(self, instance: JSON, scope: Scope) -> None:
         if len(instance) == 0:
@@ -90,8 +88,8 @@ class ItemsKeyword_2019_09(Keyword, Applicator, ArrayApplicator):
 
 class AdditionalItemsKeyword_2019_09(Keyword, Applicator):
     key = "additionalItems"
-    types = "array"
-    depends = "items"
+    types = "array",
+    depends = "items",
 
     def evaluate(self, instance: JSON, scope: Scope) -> None:
         if (items := scope.sibling(instance, "items")) and type(items.annotation) is int:
@@ -108,8 +106,8 @@ class AdditionalItemsKeyword_2019_09(Keyword, Applicator):
 
 class UnevaluatedItemsKeyword_2019_09(Keyword, Applicator):
     key = "unevaluatedItems"
-    types = "array"
-    depends = "items", "additionalItems", "if", "then", "else", "allOf", "anyOf", "oneOf", "not"
+    types = "array",
+    depends = "items", "additionalItems", "if", "then", "else", "allOf", "anyOf", "oneOf", "not",
 
     def evaluate(self, instance: JSON, scope: Scope) -> None:
         last_evaluated_item = -1
